/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package l1j.server.server.model;

import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.Calendar;
import java.util.Random;

import l1j.server.Config;
import l1j.server.server.GeneralThreadPool;
import l1j.server.server.templates.L1Npc;

public class L1BossSpawn extends L1Spawn {
	private static Logger _log = Logger.getLogger(L1BossSpawn.class.getName());
	private String _cycleType;
	private int _percentage;
	private L1BossCycle _cycle;
	private int _spawnCount;
	private Calendar _activeSpawnTime;
	private static Random _rnd = new Random();
	private int _spawnNumber;
	private int _objectId;
	
	private class SpawnTask implements Runnable {

		private SpawnTask(int spawnNumber, int objectId) {
			_spawnNumber = spawnNumber;
			_objectId = objectId;
		}

		@Override
		public void run() {
			doSpawn(_spawnNumber, _objectId);
		}
	}

	public L1BossSpawn(L1Npc mobTemplate) {
		super(mobTemplate);
	}

	/**
	 * SpawnTask To start.
	 * @param spawnNumber L1Spawn Which is administered by the numbers. Home to specify what point does not exist and it's good.
	 */
	@Override
	public void executeSpawnTask(int spawnNumber, int objectId) {
		// count Decrement to check the entire death
		if (subAndGetCount() != 0) {
			return; // Not all dead
		}
		// Last appearance time that the next time to calculate the emergence
		Calendar spawnTime;
		Calendar now = Calendar.getInstance(); 
		Calendar latestStart = _cycle.getLatestStartTime(now); 
		Calendar activeStart = _cycle.getSpawnStartTime(_activeSpawnTime); 
		// Period of time and was active in the recent period started started the same time, the emergence of
		if (!activeStart.before(latestStart)) {
			spawnTime = calcNextSpawnTime(activeStart);
		} else {
			// If different from the one before the emergence of a period of
			latestStart.add(Calendar.SECOND, -1);
			spawnTime = calcNextSpawnTime(_cycle.getLatestStartTime(latestStart));
		}
		spawnBoss(spawnTime, objectId);
	}

	private synchronized int subAndGetCount() {
		return --_spawnCount;
	}

	public void setCycleType(String type) {
		_cycleType = type;
	}

	public void setPercentage(int percentage) {
		_percentage = percentage;
	}

	@Override
	public void init() {
		if (_percentage <= 0) {
			return;
		}
		_cycle = L1BossCycle.getBossCycle(_cycleType);
		if (_cycle == null) {
			throw new RuntimeException(_cycleType + " not found");
		}
		Calendar now = Calendar.getInstance();
		// Appearance time
		Calendar spawnTime;
		if (Config.INIT_BOSS_SPAWN && _percentage > _rnd.nextInt(100)) {
			spawnTime = _cycle.calcSpawnTime(now);

		} else {
			spawnTime = calcNextSpawnTime(now);
		}
		spawnBoss(spawnTime, 0);
	}

	// Next calculate the probability of the emergence of time to calculate
	private Calendar calcNextSpawnTime(Calendar cal) {
		do {
			cal = _cycle.nextSpawnTime(cal);
		} while (!(_percentage > _rnd.nextInt(100)));
		return cal;
	}

	// The boss at the time specified in the emergence of schedule
	private void spawnBoss(Calendar spawnTime, int objectId) {
		// The emergence of time to keep. When re-emergence.
		_activeSpawnTime = spawnTime;
		long delay = spawnTime.getTimeInMillis() - System.currentTimeMillis();
		int cnt = _spawnCount;
		_spawnCount = getAmount();
		while (cnt < getAmount()) {
			cnt++;
			GeneralThreadPool.getInstance().schedule(new SpawnTask(0, objectId), delay);
		}
		_log.log(Level.FINE, toString());
	}

	/**
	 * The boss of the current active period of time and appearance, respectively.
	 */
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("[MOB]npcid:" + getNpcId());
		builder.append(" name:" + getName());
		builder.append("[Type]" + _cycle.getName());
		builder.append("[The current cycle]");
		builder.append(_cycle.getSpawnStartTime(_activeSpawnTime).getTime());
		builder.append(" - ");
		builder.append(_cycle.getSpawnEndTime(_activeSpawnTime).getTime());
		builder.append("[Appearance time]");
		builder.append(_activeSpawnTime.getTime());
		return builder.toString();
	}
}